/* DAW to CV USB interface
 *  
 * The sketch reads up to 7 MIDI Control Change events from USB (i.e. coming 
 * from your DAW) and translates them into Control Voltages.
 * Remember to set the MIDI_RECEIVE_CHANNEL of your interest.
 * 
 * Hardware: ATmega 32u4 board (i.e. Arduino Leonardo) and a RC low pass 
 * filter for each output (R = 1Kohm, C = 0.1uF).
 * 
 * Requires MIDIUSB library to be installed to compile!
 * 
 * Arduino Leonardo PWM info:
 * 
 * Pins 3, 11: controlled by timer 0 (WARNING: this timer controls millis() and delay() functions, so it's better to don't touch it)
 * Pins 9, 10: controlled by timer 1
 * Pin 5     : controlled by timer 3
 * Pin 6, 13 : controlled by timer 4
 * 
 * Setting    Divisor    Frequency
 * 0x01        1         31250
 * 0x02        8         3906.25
 * 0x03        64        488.28125
 * 0x04        256       122.0703125
 * 0x05        1024      30.517578125
 * 
 * TCCR0B = TCCR0B & 0b11111000 | <setting>; //sets the frequency for timer 0, pins 3 and 11
 * TCCR1B = TCCR1B & 0b11111000 | <setting>; //sets the frequency for timer 1, pins 9 and 10
 * TCCR3B = TCCR3B & 0b11111000 | <setting>; //sets the frequency for timer 3, pin 5
 * TCCR4B = TCCR4B & 0b11111000 | <setting>; //sets the frequency for timer 4, pins 6 and 13
 * 
 * by Barito 2019-2020
 */

#include "MIDIUSB.h"

#define MIDI_RECEIVE_CHANNEL 1 //set at your will (1-16)
#define PWM 7

const byte CC_Pin[PWM] = {3, 5, 6, 9, 10, 11, 13};//array of Leonardo pwm pins 
const byte CC[PWM] =  {19, 20, 21, 22, 23, 24 ,25};//array of CC messages to respond to
byte basic_channel; //0-15

void setup() {
for (byte i = 0; i<PWM; i++){
  pinMode(i, OUTPUT);
  //Serial.begin(115200);
}
TCCR0B = TCCR0B & 0b11111000 | 0x01; //sets the frequency to 31250 Hz for timer 0, pins 3 and 11
TCCR1B = TCCR1B & 0b11111000 | 0x01; //sets the frequency to 31250 Hz for timer 1, pins 9 and 10
TCCR3B = TCCR3B & 0b11111000 | 0x01; //sets the frequency to 31250 Hz for timer 3, pin 5
TCCR4B = TCCR4B & 0b11111000 | 0x01; //sets the frequency to 31250 Hz for timer 4, pins 6 and 13
}

void loop() {
  midiEventPacket_t rx;
  do {
    rx = MidiUSB.read();
    if (rx.header != 0) {
    // rx.header is the event type (0x0B = control change).
    // rx.byte1 is the event type, combined with the channel.
    // rx.byte2 is the CC number (0-119).
    // rx.byte3 is the CC value (0-127).
        if(rx.header == 0x0B){
          basic_channel = 0xB0 ^ rx.byte1;
          if(basic_channel == MIDI_RECEIVE_CHANNEL-1){
            for(int i = 0; i< PWM; i++){
              if(rx.byte2 == CC[i]){
                  analogWrite(CC_Pin[i], rx.byte3<<1);
              }
            }
          }
        }
      }
    } 
   while (rx.header != 0);
}
